<?php
if (!isset($siledap_path)) {
	require_once("../../../config.php");
}
require_once($urnreg_path."/common.php");
require_once($urnreg_path."/AuthREngine.php");
require_once($siledap_path."/LdapConf.php");
require_once($siledap_path."/LdapManager.php");

$URN_PENDING='urn:mace:terena.org:schac:userStatus:edugain:registry:pending';

$error='';

function uuid() {
    
    // The field names refer to RFC 4122 section 4.1.2

    return sprintf('%04x%04x-%04x-%03x4-%04x-%04x%04x%04x',
        mt_rand(0, 65535), mt_rand(0, 65535), // 32 bits for "time_low"
        mt_rand(0, 65535), // 16 bits for "time_mid"
        mt_rand(0, 4095),  // 12 bits before the 0100 of (version) 4 for "time_hi_and_version"
        bindec(substr_replace(sprintf('%016b', mt_rand(0, 65535)), '01', 6, 2)),
            // 8 bits, the last two of which (positions 6 and 7) are 01, for "clk_seq_hi_res"
            // (hence, the 2nd hex digit after the 3rd hyphen can only be 1, 5, 9 or d)
            // 8 bits for "clk_seq_low"
        mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535) // 48 bits for "node"  
    );  
}

function searchSubURNs($lm,$urn,$search) {
	GLOBAL $URN_LDAP_BASE;
	GLOBAL $OID_LDAP_BASE;

	$searchBase = $URN_LDAP_BASE;
	if (strpos($urn, "urn:oid:")!==FALSE && strpos($urn, "urn:oid:")==0) {
		$searchBase = $OID_LDAP_BASE;
	}
	
	$resState=true;
	
	$filter="(urnRegValue=".$urn.":*)";
	$entries=$lm->search($searchBase,$filter,false,array("objectClass","urnRegValue"));
	usort($entries,"sortSubUrns");
	if (count($entries)>0) {
		$entryMin=$entries[0];
		$min=count(explode(":",$entryMin->getSingleAttribute("urnRegValue")));
		$end=false;
		for ($i=0; $i<count($entries)&&$end==false&&$resState==true; $i++) {
			$entry=$entries[$i];
			if ($entry->hasAttribute("schacUserStatus")==false) {
				$tokens=count(explode(":",$entry->getSingleAttribute("urnRegValue")));
				if ($tokens>$min) {
					$end=true;
				}
				else {
					$objs=$entry->getObjectClassList()->getValues();
					$urnvalue=$entry->getSingleAttribute("urnRegValue");
					if (in_array("urnRegDelegationStr",$objs)||in_array("urnRegDelegation",$objs)) {
						$wsEP=$entry->getSingleAttribute("urnRegHttpsURI");
						$resState=searchURNs($lm,$urnvalue,$search,$wsEP);
					}
					else if (in_array("urnRegDescriptionStr",$objs)||in_array("urnRegDescription",$objs)) {
						$resState=searchURNs($lm,$urnvalue,$search);
					}
				}
			}
		}
	}
	
	return $resState;
}

function searchURNs($lm,$urn,$search,$wsEP='') {
	$resState=false;
	
	$searchUrn = $urn;
	if (strpos($searchUrn, "urn:")===FALSE) {
		$searchUrn = "urn:oid:".implode(":",explode(".",$urn));
	}
	
	error_log($searchUrn);
	
	if (strcmp($wsEP,"")==0) { // It's NOT a delegated URN
		$urnEntry=getURNEntry($lm,$searchUrn);
		if (!is_string($urnEntry)) {
			$objs=$urnEntry->getObjectClassList()->getValues();
			$urnvalue=$urnEntry->getSingleAttribute("urnRegValue");
			if (in_array("urnRegDelegationStr",$objs)||in_array("urnRegDelegation",$objs)) { // Anyway, we should check if it's a delegated URN
				$newWsEP=$urnEntry->getSingleAttribute("urnRegHttpsURI");
				$resState=searchURNs($lm,$urnvalue,$search,$newWsEP); // Searching again with its end point
			}
			else if (in_array("urnRegDescriptionStr",$objs)||in_array("urnRegDescription",$objs)) { // Actually, it's a description urn
				$urndescs=$urnEntry->getSingleAttribute("urnRegValueDescription");
				$found=false;
				foreach ($urnEntry->getAttribute("urnRegValueDescription")->getArrayValues() as $descattr) {
					$found|=stripos($descattr['value'],$search)!==false;
				}
				if (strpos($urnvalue, "urn:oid:")!==FALSE && strpos($urnvalue, "urn:oid:")==0) {
					$found|=stripos(implode(".",explode(":",substr($urnvalue,8))),$search)!==false;
				}
				else {
					$found|=stripos($urnvalue,$search)!==false;
				}
				if ($found==true) {
					$user = "";
					if ($urnEntry->hasAttribute("seeAlso")) {
						$user = $lm->getEntry($urnEntry->getSingleAttribute("seeAlso"));
					}
					echo echoURNDescription($urnEntry, $user);
				}
				$resState=true;
			}
			// Search in sub URNs
			if ($resState==true) {
				$resState=searchSubURNs($lm,$urnvalue,$search);		
			}
		}
	}
	else {
		$resState=true;
	
		$urnxml=getXMLDelegatedURN($searchUrn,$wsEP);
		if (strcmp($urnxml,"")!=0) {
	    	$xml=new SimpleXMLElement($urnxml);
			$found=stripos($xml['value'],$search)!==false;
		    foreach ($xml->descriptions->description as $desc) {
				$found|=stripos($desc['value'],$search)!==false;
    		}
			if ($found==true) {
				$xmlArray = explode("\n",$xml->asXML());
				if (strpos($xmlArray[0],"xml")!==FALSE) {
					array_shift($xmlArray);
				}
				$xmlstring = implode("\n",$xmlArray);
				echo $xmlstring;
			}
			$children=getArrayDelegatedSubURNs($xml['value'],$wsEP);
			foreach ($children as $child) {
				searchURNs($lm,$child['value'],$search,$wsEP);
			}
		}
	}
	
	return $resState;
}

function getArrayDelegatedSubURNs($urnvalue,$wsEP) {
	$params="?format=machine";
	
	$content='';
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $wsEP."/list/".$params);
    curl_setopt($ch, CURLOPT_HEADER, 0); 
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $content);
    ob_start();
    curl_exec ($ch);
    curl_close ($ch);
    $res = ob_get_contents();
    ob_end_clean();

	$elems=array();
	$temp=explode(":",$urnvalue);
	$numtokens=count($temp);
    $xml=new SimpleXMLElement($res);
	foreach ($xml->urns->urn as $urn) {
		$value=$urn['value'];
		if (strpos($value,utf8_encode($urnvalue))!==FALSE) {
			$temp=explode(":",$value);
			if ((count($temp)-1)==$numtokens) {
	    		$urn->addAttribute("delegated",$wsEP);
    			$elems[]=$urn;
			}
		}
	}
	return $elems;
}

function getDelegatedSubURNs($urnvalue,$wsEP) {
	$res=getArrayDelegatedSubURNs($urnvalue,$wsEP);

	$ret=true;
	foreach ($res as $elem) {
		echo $elem->asXML();
	}
	return $ret;
}

function addUrnDel($lm,$dataXML) {
	GLOBAL $URN_PENDING;
	GLOBAL $URN_VALUE_BASE;
	GLOBAL $USERS_LDAP_BASE;
	GLOBAL $error;
	GLOBAL $URN_LDAP_BASE;
	GLOBAL $OID_LDAP_BASE;

	$searchBase = $URN_LDAP_BASE;
	if (strpos($searchBase, "urn:oid:")!==FALSE && strpos($searchBase, "urn:oid:")==0) {
		$searchBase = $OID_LDAP_BASE;
	}
	
	$xml = simplexml_load_string($dataXML);
	
	$objectClasses = array('urnRegDelegationStr','irisObject','top','schacUserEntitlements');
	$uuid=uuid();
	$attrs = array();
	$address=base64_decode($xml->urn_data->contact_person->postal_address['value']);
	$tokenAddress=explode("\n",$address);
	$address=implode(" $ ",$tokenAddress);
	$address.=" $ ".base64_decode($xml->urn_data->contact_person->country['value']);
	
	$urnvalue=$xml->urn_data['value'];

	$authrEngine = new AuthREngine();
	$admins = $authrEngine->getAdmins($urnvalue);
	
	$parenturnA=explode(":",$urnvalue);
	array_pop($parenturnA);
	$parenturn=implode(":",$parenturnA);
	$valueparenturn=getURNEntry($lm,$parenturn);
	
	$existPrefix=false;
	foreach($URN_VALUE_BASE as $urnprefix) {
		if (strpos($urnvalue,$urnprefix)!==false) {
			$existPrefix=true;
		}
	}

	if (is_string($valueparenturn)) {
		$error="The parent URN ".$parenturn." has not been registered";
		return false;
	}
	else if (strcmp($urnvalue,"")==0||$existPrefix==false) {
		$error="The URN is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->access_data->regwebapp['value'],"")==0||strcmp(base64_decode($xml->urn_data->access_data->regwebapp['value']),"")==0||(!(strpos(base64_decode($xml->urn_data->access_data->regwebapp['value']),"https://")===0))) {
		$error="The register web application is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->access_data->policy['value'],"")==0||strcmp(base64_decode($xml->urn_data->access_data->policy['value']),"")==0||(!(strpos(base64_decode($xml->urn_data->access_data->policy['value']),"https://")===0))) {
		$error="The policy document is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->access_data->webservices['value'],"")==0||strcmp(base64_decode($xml->urn_data->access_data->webservices['value']),"")==0||(!(strpos(base64_decode($xml->urn_data->access_data->webservices['value']),"https://")===0))) {
		$error="The web services URL is not valid";
		return false;
	}
	else if ((count($xml->urn_data->access_data->ldap)>0)&&(!(strcmp($xml->urn_data->access_data->ldap['value'],"")!=0&&strcmp(base64_decode($xml->urn_data->access_data->ldap['value']),"")!=0&&strpos(base64_decode($xml->urn_data->access_data->ldap['value']),"ldap")===0))) {
		$error="The LDAP URI is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->domain['value'],"")==0||strcmp(base64_decode($xml->urn_data->contact_person->domain['value']),"")==0) {
		$error="The domain of the contact person is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->cn['value'],"")==0||strcmp(base64_decode($xml->urn_data->contact_person->cn['value']),"")==0) {
		$error="The common name of the contact person is not valid";
		return false; 
	}
	else if (strcmp($xml->urn_data->contact_person->email['value'],"")==0||strcmp(base64_decode($xml->urn_data->contact_person->email['value']),"")==0) {
		$error="The email of the contact person is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->sn1['value'],"")==0||strcmp(base64_decode($xml->urn_data->contact_person->sn1['value']),"")==0) {
		$error="The first surname of the contact person is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->given_name['value'],"")==0||strcmp(base64_decode($xml->urn_data->contact_person->given_name['value']),"")==0) {
		$error="The given name of the contact person is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->postal_address['value'],"")==0||strcmp(base64_decode($xml->urn_data->contact_person->postal_address['value']),"")==0) {
		$error="The postal address of the contact person is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->country['value'],"")==0||strcmp(base64_decode($xml->urn_data->contact_person->country['value']),"")==0) {
		$error="The country of the contact person is not valid";
		return false;
	}
	$attrs['idnc']=array(utf8_encode($uuid));
	$attrs['urnRegValue']=array(utf8_encode($xml->urn_data['value']));
	$attrs['urnRegHttpURIHuman']=array(utf8_encode(base64_decode($xml->urn_data->access_data->regwebapp['value'])));
	$attrs['urnRegPolicyURI']=array(utf8_encode(base64_decode($xml->urn_data->access_data->policy['value'])));
	$attrs['urnRegHttpURIMachine']=array(utf8_encode(base64_decode($xml->urn_data->access_data->webservices['value'])));
	if (count($xml->urn_data->access_data->ldap)>0) {
		$ldapdata=base64_decode($xml->urn_data->access_data->ldap['value']);
		if (strcmp($ldapdata,"ldap://")!=0) {
			$attrs['urnRegLdapURI']=array(utf8_encode($ldapdata));
		}
	}
	$attrs['urnRegOwner']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->domain['value'])));
	$attrs['schacUserStatus']=array($URN_PENDING);
	$uuidUser=uuid();
	$attrs['seeAlso']=array($uuidUser);

	$res=$lm->addEntry("idnc=".$uuid.",".$searchBase,$objectClasses,$attrs);

	$objectClasses = array('urnRegContactPersonStr','irisObject','top','schacUserEntitlements');
	$attrs = array();
	$attrs['idnc']=array(utf8_encode($uuidUser));
	$attrs['cn']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->cn['value'])));
	$attrs['givenName']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->given_name['value'])));
	$attrs['mail']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->email['value'])));
	$attrs['postalAddress']=array(utf8_encode($address));
	$attrs['schacSn1']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->sn1['value'])));
	if (strcmp($xml->urn_data->contact_person->sn2['value'],"AA==")!=0)
		$attrs['schacSn2']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->sn2['value'])));
	if (strcmp($xml->urn_data->contact_person->phone['value'],"AA==")!=0)
		$attrs['telephoneNumber']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->phone['value'])));
	$res=$lm->addEntry("idnc=".$uuidUser.",".$USERS_LDAP_BASE,$objectClasses,$attrs);

	if ($res == true) {
		sendUrnRegEmail($dataXML, implode(",",$admins));
	}
	
	return $res;
}

function addUrnReg($lm,$dataXML) {
	GLOBAL $URN_PENDING;
	GLOBAL $URN_VALUE_BASE;
	GLOBAL $OID_VALUE_BASE;
	GLOBAL $USERS_LDAP_BASE;
	GLOBAL $error;
	GLOBAL $URN_LDAP_BASE;
	GLOBAL $OID_LDAP_BASE;

	$searchBase = $URN_LDAP_BASE;
	$valueBase = $URN_VALUE_BASE;
	$mode = 'urn';

	$xml = simplexml_load_string($dataXML);
	
	$objectClasses = array('urnRegDescriptionStr','irisObject','top','schacUserEntitlements');
	$uuid=uuid();
	$attrs = array();
	$address=base64_decode($xml->urn_data->contact_person->postal_address['value']);
	$tokenAddress=explode("\n",$address);
	$address=implode(" $ ",$tokenAddress);
	$address.=" $ ".base64_decode($xml->urn_data->contact_person->country['value']);
	
	$urnvalue=$xml->urn_data['value'];
	if (strpos($urnvalue, "urn:oid:")!==FALSE && strpos($urnvalue, "urn:oid:")==0) {
		$searchBase = $OID_LDAP_BASE;
		$valueBase = $OID_VALUE_BASE;
		$mode = 'oid';
	}

	$authrEngine = new AuthREngine();
	$admins = $authrEngine->getAdmins($urnvalue);
	
	$parenturnA=explode(":",$urnvalue);
	array_pop($parenturnA);
	$parenturn=implode(":",$parenturnA);
	$valueparenturn=getURNEntry($lm,$parenturn);
	
	$existPrefix=false;
	foreach($valueBase as $urnprefix) {
	
		if (strcmp($mode,"oid")==0&&strpos($urnvalue,"urn:oid:".implode(":",explode(".",$urnprefix)))!==false) {
			$existPrefix=true;
		}
		else if (strcmp($mode,"urn")==0&&strpos($urnvalue,$urnprefix)!==false) {
			$existPrefix=true;
		}
	}
	if (is_string($valueparenturn)) {
		$error="The parent URN ".$parenturn." has not been registered";
		return false;
	}
	else if (strcmp($urnvalue,"")==0||$existPrefix==false) {
		$error="The URN is not valid";
		return false;
	}
	else if (strcmp(base64_decode($xml->urn_data->description['value']),"")==0) {
		$error="The description of the URN is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->cn['value'],"")==0) {
		$error="The common name of the contact person is not valid";
		return false; 
	}
	else if (strcmp($xml->urn_data->contact_person->email['value'],"")==0) {
		$error="The email of the contact person is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->sn1['value'],"")==0) {
		$error="The first surname of the contact person is not valid";
		return false;
	}
	else if (strcmp($xml->urn_data->contact_person->given_name['value'],"")==0) {
		$error="The given name of the contact person is not valid";
		return false;
	}
	else if (strcmp(base64_decode($xml->urn_data->contact_person->postal_address['value']),"")==0) {
		$error="The postal address of the contact person is not valid";
		return false;
	}
	else if (strcmp(base64_decode($xml->urn_data->contact_person->country['value']),"")==0) {
		$error="The country of the contact person is not valid";
		return false;
	}
	else if (is_string(getURNEntry($lm,$urnvalue))==false) {
		$error="The URN ".$urnvalue." is already registered or is still pending approval. ";
		return false;
	}
	$attrs['idnc']=array(utf8_encode($uuid));
	$attrs['urnRegValue']=array(utf8_encode($xml->urn_data['value']));
	$attrs['urnRegValueDescription']=array(utf8_encode(base64_decode($xml->urn_data->description['value'])));
	if (count($xml->urn_data->presentations->presentation)>0) {
		foreach ($xml->urn_data->presentations->presentation as $pres) {
			$attrs['urnRegValuePresentation;lang-'.$pres['lang']]=array(utf8_encode(base64_decode($pres['value'])));
		}
	}
	$attrs['schacUserStatus']=array($URN_PENDING);
	$uuidUser=uuid();
	$attrs['seeAlso']=array("idnc=".$uuidUser.",".$USERS_LDAP_BASE);
	
	$res=$lm->addEntry("idnc=".$uuid.",".$searchBase,$objectClasses,$attrs);

	if ($res==true) {
		$objectClasses = array('urnRegContactPersonStr','irisObject','top','schacUserEntitlements');
		$attrs = array();
		$attrs['idnc']=array(utf8_encode($uuidUser));
		$attrs['cn']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->cn['value'])));
		$attrs['givenName']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->given_name['value'])));
		$attrs['mail']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->email['value'])));
		$attrs['postalAddress']=array(utf8_encode($address));
		$attrs['schacSn1']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->sn1['value'])));
		if (strcmp($xml->urn_data->contact_person->sn2['value'],"AA==")!=0)
			$attrs['schacSn2']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->sn2['value'])));
		if (strcmp($xml->urn_data->contact_person->phone['value'],"AA==")!=0)
			$attrs['telephoneNumber']=array(utf8_encode(base64_decode($xml->urn_data->contact_person->phone['value'])));
		
		$res=$lm->addEntry("idnc=".$uuidUser.",".$USERS_LDAP_BASE,$objectClasses,$attrs);
		
		if ($res == true) {
			sendUrnRegEmail($dataXML, implode(",",$admins));
		}
	}	
	return $res;
}

function sendUrnRegEmail($dataXML, $bcc) {
	GLOBAL $EMAIL_SENDER;
	GLOBAL $EMAIL_SUBJECT;
	GLOBAL $EMAIL_TEXT;

	$xml = simplexml_load_string($dataXML);
	
	$urn = $xml->urn_data['value'];
	$to = base64_decode($xml->urn_data->contact_person->email['value']);
	$subject = str_replace("%%URN_VALUE%%",$urn,$EMAIL_SUBJECT);
	$text = $EMAIL_TEXT;
	$headers = 'From: ' . $EMAIL_SENDER . "\r\n" .
    'Bcc: ' . $bcc . "\r\n" .
    'Reply-To: ' . $EMAIL_SENDER . "\r\n" .
    'X-Mailer: PHP/' . phpversion();
	mail($to,$subject,$text,$headers);
}

function searchPendingURNs($lm) {
	GLOBAL $URN_VALUE_BASE;
	GLOBAL $URN_LDAP_BASE;
	GLOBAL $OID_LDAP_BASE;
	GLOBAL $OID_VALUE_BASE;

	$searchBase = array(array($URN_LDAP_BASE,$URN_VALUE_BASE),array($OID_LDAP_BASE,$OID_VALUE_BASE));
	$resState=true;

	foreach ($searchBase as $search) {	
		$ldapBase = $search[0];
		$dataBase = $search[1];
		foreach ($dataBase as $urnparent) {
			$searchurn = $urnparent;
			if (!(strpos($searchurn, "urn:")!==FALSE && strpos($searchurn, "urn:")==0)) {
				$searchurn = "urn:oid:".implode(":",explode(".",$searchurn));
			}
			$filter="(&(urnRegValue=".$searchurn.":*)(schacUserStatus=*))";
			error_log($filter);
			$entries=$lm->search($ldapBase,$filter,false);
			usort($entries,"sortSubUrns");
			for ($i=0; $i<count($entries); $i++) {
				$entry=$entries[$i];
				$urnvalue=$entry->getSingleAttribute("urnRegValue");
				
				$authrEngine = new AuthREngine();
				if ($authrEngine->isAllowedToAdmin($urnvalue)) {
				
					$tokens=count(explode(":",$entry->getSingleAttribute("urnRegValue")));
					$objs=$entry->getObjectClassList()->getValues();
					$user = "";
					if ($entry->hasAttribute("seeAlso")) {
						$user = $lm->getEntry($entry->getSingleAttribute("seeAlso"));
					}
					if (in_array("urnRegDescriptionStr",$objs)||in_array("urnRegDescription",$objs)) {
						echo echoURNDescription($entry, $user);
						$resState=true;
					}
					else if (in_array("urnRegDelegationStr",$objs)||in_array("urnRegDelegation",$objs)) {
						echo echoURNDelegation($entry, $user);
						$resState=true;
					}
				
				}
			}
		}
	}

	return $resState;
}

$lm=new LdapManager($LDAP_SERVER_NAME,$LDAP_SERVER_PORT);
$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
$lm->connectLdap($LDAP_USER,$LDAP_PASS);

$success=false;

header('Content-type: text/xml');
echo "<Response>";
if (array_key_exists('oper',$_REQUEST)) {
   	$oper=$_REQUEST['oper'];
    if (strcmp($oper,"geturn")==0) { // Get an urn
        if (array_key_exists('urnvalue',$_REQUEST)) {
        	$urnvalue=rawurldecode($_REQUEST['urnvalue']);
        	if (array_key_exists('delegated',$_REQUEST)) {
        		$urndata=getDelegatedURN($urnvalue,rawurldecode($_REQUEST['delegated']));
        		if (strcmp($urndata,"")!=0) {
        			echo $urndata;
        		}
        		else {
        			$sucess=false;
        			$error="URN ".$urnvalue." not found";
        		}
        	}
        }
    }
    else if (strcmp($oper,"getsuburns")==0) { // Get sub urns
        if (array_key_exists('urnvalue',$_REQUEST)) {
        	$urnvalue=rawurldecode($_REQUEST['urnvalue']);
        	if (array_key_exists('delegated',$_REQUEST)) {
	        	$urndata=getDelegatedSubURNs($urnvalue,rawurldecode($_REQUEST['delegated']));
        		if (strcmp($urndata,"")!=0) {
        			echo $urndata;
        		}
        		else {
        			$sucess=false;
        			$error="URN ".$urnvalue." not found";
        		}
        	}
        }
    }
	else if (strcmp($oper,"search")==0) { // Search urns
		if (array_key_exists('search',$_REQUEST)) {
			$search=rawurldecode($_REQUEST['search']);
			foreach ($OID_VALUE_BASE as $oidbase) {
				$urnbase="urn:oid:".implode(":",explode(".",$oidbase));
				$success=searchURNs($lm,$urnbase,$search);
			}
			foreach ($URN_VALUE_BASE as $urnbase) {
				$success=searchURNs($lm,$urnbase,$search);
			}
		}
	}
	else if (strcmp($oper,"addurn")==0) { // Request a new urn
		if (array_key_exists('urndata',$_REQUEST)) {
			$urntype=rawurldecode($_REQUEST['urntype']);
			$data=base64_decode($_REQUEST['urndata']);
			if (strcmp($urntype,"urnreg")==0) {
				$success=addUrnReg($lm,$data);
			}
			else {
				$success=addUrnDel($lm,$data);
			}
		}
	}
	else if (strcmp($oper,"getpendingurn")==0) { // Get pendings URNs
		$success=searchPendingURNs($lm);
	}
}
if ($success==true) {
	echo "<OperationInfo><Status status='success'/></OperationInfo>";
}
else {
	if (strcmp($error,"")==0) {
		$error=$lm->getErrorMessage();
	}
	echo "<OperationInfo><Status status='fail'/><ErrorMessage message='".$error."'/></OperationInfo>";
}
echo "</Response>";

?>
